function [bestR2,bestMIdx,bestPwrs,bestCoeffs,mdlBest,mdlAll] ...
  = ebrm2Small(xdata,ydata,xlow,xstep,xhi)
  % handles relatively small data sets
  warning("off")
  fprintf("Please wait ....\n");
  [S,xdata,ydata]= Init(xdata,ydata,xlow,xstep,xhi);
  % shuffle array
  idx = randperm(S.N);
  x = xdata(idx,:);
  y = ydata(idx); 
  dSize = fix(0.8*S.N);
  x = x(1:dSize,:);
  y = y(1:dSize);
  bestR2a = 0;
  for i=1:S.M
    pArr = S.pwrMat(i,1:S.V+1);
    py = pArr(1); 
    px1 = pArr(2); 
    px2 = pArr(3); 
    eval(S.sMdlX);
    eval(S.sMdlY);
    mdl = fitlm(X,yt);
    r2a = mdl.Rsquared.Adjusted;
    if isnan(r2a) || isinf(r2a) || r2a < 0 || r2a > 1
      r2a = 0; 
    end
    if bestR2a < r2a
      bestR2a = r2a;
      bestMdl = mdl;
      bestpArr = pArr;
    end
    S.stats(i,1:S.V+2) = ...
      [mdl.Coefficients{1,1}, ...
       mdl.Coefficients{2,1}, ...
       mdl.Coefficients{3,1}, ...
       r2a];
  end
  fprintf("for Bin 1\nBest powers: ")
  fprintf("%f, ", bestpArr);
  fprintf("\nBest Model:\n")
  bestMdl
  anova(bestMdl,"summary")
  fprintf("Finished calculations with Bin 1\n\n")
  cr = size(S.V+1,1);
  for iBin=2:S.B
    % shuffle array
    idx = randperm(S.N);
    x = xdata(idx,:);
    y = ydata(idx); 
    x = x(1:dSize,:);
    y = y(1:dSize);
    for i=1:S.M
      pArr = S.pwrMat(i,1:S.V+1);
      py = pArr(1);
      px1 = pArr(2);
      px2 = pArr(3);
      for k=1:S.V+1
        cr(k) = S.stats(i,k);
      end
      eval(S.sEvalX);
      eval(S.sEvalY);
      % yhat=yhat';
      e = y-yhat;
      SSres = sum(dot(e,e));
      ymean = mean(y);
      e = y-ymean;
      SStot = sum(dot(e,e));
      r2 = max(0,1 - SSres/SStot);
      if r2 > 0
        r2adj = 1 - (1-r2)*(S.N-1)/(S.N - S.V - 1);
      else
        r2adj = 0;
      end
      S.Bin(iBin,i) = r2adj;
      S.Bin(1,i) = S.Bin(1,i) + S.Bin(iBin,i);
    end       
    [bestR2,bestMIdx] = max(S.Bin(iBin,:));
    bestPwrs = S.pwrMat(bestMIdx,:);
    fprintf("Bin # %d\n", iBin)
    fprintf("Best R2adj = %f\n", bestR2)
    fprintf("Best model # %d\n", bestMIdx)
    fprintf("Powers: ")
    fprintf(" %f, ", bestPwrs)
    fprintf("\n")

  end
  fprintf("\n\n")
  S.Bin(1,:) = S.Bin(1,:)/(S.B-1);
  [bestR2,bestMIdx] = max(S.Bin(1,:));
  bestPwrs = S.pwrMat(bestMIdx,:);
  bestCoeffs = S.stats(bestMIdx,1:S.V+1);    
  x = xdata(1:dSize,:);
  y = ydata(1:dSize);
  py = bestPwrs(1); 
  px1 = bestPwrs(2); 
  px2 = bestPwrs(3); 
  eval(S.sMdlX);
  eval(S.sMdlY);
  mdlBest = fitlm(X,yt);
  fprintf("Using data in ALL bins -----------\n")
  x = xdata;
  y = ydata;
  eval(S.sMdlX);
  eval(S.sMdlY);
  mdlAll = fitlm(X,yt);
end

function y = fxt(x,pwr)
  if pwr > 0
    y = x.^pwr;
  elseif pwr < 0
    y = 1./x.^abs(pwr);
  else
    y = log(x);
  end
end

function y = fxinv(x,pwr)
  if pwr > 0
    y = x.^(1/pwr);
  elseif pwr < 0
    y = x.^abs(pwr);
  else
    y = exp(x);
  end
end


function [S,x,y]= Init(x,y,xlow,xstep,xhi)
  [rows,cols] = size(x);
  S.N = rows;
  S.V = cols;
  S.B = 4;

  S.sMdlX = "X = [fxt(x(:,1),px1) fxt(x(:,2),px2)];";
  S.sMdlY = "yt = fxt(y,py);";
  S.sEvalX = "yhat=cr(1)+cr(2)*fxt(x(:,1),px1)+cr(3)*fxt(x(:,2),px2);";
  S.sEvalY = "yhat = fxinv(yhat,1/py);";
  % count the total number of models
  count=0;
  for py=xlow(S.V+1):xstep(S.V+1):xhi(S.V+1)
    for px1=xlow(1):xstep(1):xhi(1)
      for px2=xlow(2):xstep(2):xhi(2)
        count=count+1; 
      end
    end
  end  
  S.M = count;

  S.pwrMat = zeros(S.M,S.V+1);
  S.stats= zeros(S.M,S.V+2);
  i=0;
  for py=xlow(S.V+1):xstep(S.V+1):xhi(S.V+1)
    for px1=xlow(1):xstep(1):xhi(1)
      for px2=xlow(2):xstep(2):xhi(2)
        i=i+1;
        S.pwrMat(i,1:S.V+1)=[py,px1,px2];
        % next struct maps rc(1) to rc(4) and RsqrAdj
        S.stats(i,1:S.V+2) = [0,0,0,0];
      end
    end
  end  

  % Note use S.Bin(1,) to store the average values
  % of S.Bin(2,) and up.
  S.Bin = zeros(S.B,S.M);
  
end 